@@ -87,7 +87,7 @@ class Hoe
   include Rake::DSL if defined?(Rake::DSL)
 
   # duh
-  VERSION = "3.22.1"
+  VERSION = "4.6.1"
 
   @@plugins = [:clean, :debug, :deps, :flay, :flog, :newb, :package,
                :publish, :gemcutter, :signing, :test]
@@ -114,12 +114,20 @@ class Hoe
   URLS_TO_META_MAP = {
     "bugs" => "bug_tracker_uri",
     "clog" => "changelog_uri",
+    "code" => "source_code_uri",
     "doco" => "documentation_uri",
     "docs" => "documentation_uri",
     "home" => "homepage_uri",
-    "code" => "source_code_uri",
-    "wiki" => "wiki_uri",
     "mail" => "mailing_list_uri",
+    "wiki" => "wiki_uri",
+
+    "changelog"     => "changelog_uri",
+    "changes"       => "changelog_uri",
+    "documentation" => "documentation_uri",
+    "history"       => "changelog_uri",
+    "issues"        => "bug_tracker_uri",
+    "rdoc"          => "documentation_uri",
+    "tickets"       => "bug_tracker_uri",
   }
 
   ##
@@ -162,6 +170,13 @@ class Hoe
 
   attr_accessor :description_sections
 
+  ##
+  # Optional: Add a (hopefully hidden) H2 to the description to force
+  # rubygems.org to properly render the description as rdoc. Defaults
+  # to true.
+
+  attr_accessor :description_rdoc_hack
+
   ##
   # *MANDATORY*: The author's email address(es). (can be array)
   #
@@ -181,6 +196,11 @@ class Hoe
 
   attr_accessor :extra_dev_deps
 
+  ##
+  # Optional: rdoc arguments to be used when generating rdoc.
+
+  attr_accessor :rdoc_options
+
   ##
   # Optional: Extra files you want to add to RDoc.
   #
@@ -222,6 +242,18 @@ class Hoe
 
   attr_accessor :group_name
 
+  ##
+  # Optional: The name of the executables directory. [default: bin]
+
+  attr_accessor :bindir
+
+  ##
+  # Optional: The homepage of the project. Auto-populates to the home key
+  # of the urls read from the README.txt
+  #
+
+  attr_accessor :homepage
+
   ##
   # The Gem::Specification.
 
@@ -411,7 +443,8 @@ class Hoe
     names.reject! { |n| n =~ /^[A-Z_]+$/ }
 
     names.each do |name|
-      next unless Hoe.plugins.include? name.downcase.intern
+      underscored = name.gsub(/(?<!^)(?=[A-Z])/, "_").downcase.intern
+      next unless Hoe.plugins.include? underscored
       warn "extend #{name}" if $DEBUG
       self.extend Hoe.const_get(name)
     end
@@ -446,7 +479,7 @@ class Hoe
   #  Call it multiple times if you are releasing under multiple licenses.
   #
   def license name
-    self.licenses << name
+    self.licenses << name.to_s
   end
 
   ##
@@ -475,7 +508,7 @@ class Hoe
 
     case name
     when "hoe" then
-      dependency "rake", [">= 0.8", "< 13.0"]
+      # do nothing? these deps are already in the hoe spec in the Rakefile
     else
       version = VERSION.split(/\./).first(2).join(".")
       dependency "hoe", "~> #{version}", :development
@@ -525,17 +558,18 @@ class Hoe
       s.version              = version if version
       s.summary              = summary
       s.email                = email
-      s.homepage             = urls["home"] || urls.values.first
+      s.homepage             = homepage || urls["home"] || urls.values.first
+
       s.description          = description
       s.files                = manifest
-      s.executables          = s.files.grep(/^bin/) { |f| File.basename(f) }
-      s.bindir               = "bin"
+      s.bindir               = bindir || "bin"
+      s.executables          = s.files.grep(/^#{s.bindir}/) { |f| File.basename(f) }
       s.require_paths        = dirs unless dirs.empty?
-      s.rdoc_options         = ["--main", readme_file]
+      s.rdoc_options         = rdoc_options
       s.post_install_message = post_install_message
       s.metadata             = (urls.keys & URLS_TO_META_MAP.keys).map { |name|
         [URLS_TO_META_MAP[name], urls[name]]
-      }.to_h
+      }.to_h if urls
 
       missing "Manifest.txt" if s.files.empty?
 
@@ -625,7 +659,8 @@ class Hoe
     self.author               = []
     self.changes              = nil
     self.description          = nil
-    self.description_sections = %w[description]
+    self.description_sections = %w[description features/problems]
+    self.description_rdoc_hack = true
     self.email                = []
     self.extra_deps           = []
     self.extra_dev_deps       = []
@@ -650,6 +685,9 @@ class Hoe
     self.history_file ||= Dir.glob("History.{rdoc,txt,md}").first || "History.txt"
     self.readme_file  ||= Dir.glob("README.{rdoc,txt,md}").first || "README.txt"
 
+    # delayed to set readme_file first
+    self.rdoc_options = ["--main", readme_file]
+
     abort "Hoe.new {...} removed. Switch to Hoe.spec." if block_given?
   end
 
@@ -662,18 +700,24 @@ class Hoe
                .chunk { |l| l[/^(?:=+|#+)/] || "" }
                .map(&:last)
                .each_slice(2)
-               .map { |k, v|
-                  kp = k.join
-                  kp = kp.strip.chomp(":").split.last.downcase if k.size == 1
-                  [kp, v.join.strip]
-                }
-               .to_h
+               .select { |k, v| k && v }
+               .to_h { |k, v|
+                 kp = k.join.strip.chomp(":").sub(/(?:=+|#+)\s*/, '').downcase
+
+                 # keep key in value, but title case it
+                 v.prepend k.join.downcase.gsub(/\b[a-z]/, &:upcase)
+
+                 [kp, v.join.strip]
+               }
 
     unless readme.empty? then
-      desc     = readme.values_at(*description_sections).join("\n\n")
+      desc     = readme.values_at(*description_sections).compact.join("\n\n")
+      desc     = desc.lines.drop(2).join # drop first header (eg description)
+      desc    += "\n\n==== To Install:\n" if description_rdoc_hack &&
+        desc !~ /^==+ [A-Z]/
       summ     = desc.split(/\.\s+/).first(summary_sentences).join(". ")
 
-      self.urls        ||= parse_urls(readme.values.first)
+      self.urls        ||= parse_urls(readme.values.first.lines.drop(1).join)
       self.description ||= desc
       self.summary     ||= summ
     else
@@ -708,16 +752,10 @@ class Hoe
   # should update the readme.
 
   def parse_urls text
-    lines = text.gsub(/^\* /, "").delete("<>").split(/\n/).grep(/\S+/)
+    keys  = Regexp.union Hoe::URLS_TO_META_MAP.keys
+    kv_re = /^\s*(#{keys})\s*::\s*<?([^>\s]+)>?/
 
-    if lines.first =~ /::/ then
-      Hash[lines.map { |line| line.split(/\s*::\s*/) }]
-    else
-      warn "DEPRECATED: Please switch readme to hash format for urls."
-      warn "  Only defining 'home' url."
-      warn "  This will be removed on or after 2020-10-28."
-      { "home" => lines.first }
-    end
+    text.scan(kv_re).to_h
   end
 
   ##
@@ -750,11 +788,12 @@ class Hoe
   end
 
   ##
-  # Bitch about a file that is missing data or unparsable for intuiting values.
+  # Raise about a file that is missing data or unparsable for intuiting values.
 
   def missing name
-    warn "** #{name} is missing or in the wrong format for auto-intuiting."
-    warn "   run `sow blah` and look at its text files"
+    raise \
+      "** #{name} is missing or in the wrong format for auto-intuiting." \
+      "   run `sow blah` and look at its text files"
   end
 
   ##
@@ -810,7 +849,9 @@ class Hoe
 
   def post_initialize
     activate_plugin_deps
-    intuit_values File.read_utf readme_file if readme_file
+    unless skip_intuit_values?
+      intuit_values File.read_utf readme_file if readme_file
+    end
     validate_fields
     define_spec
     load_plugin_tasks
@@ -833,6 +874,8 @@ class Hoe
     spec_extras[:required_rubygems_version] = versions
   end
 
+  alias required_rubygems_version require_rubygems_version
+
   ##
   # Declare that your gem requires a specific ruby version. Last one wins.
 
@@ -840,20 +883,6 @@ class Hoe
     spec_extras[:required_ruby_version] = versions
   end
 
-  ##
-  # Declare that this gem requires ruby to be in the 1.8+ family.
-
-  def ruby18!
-    require_ruby_version "~> 1.8"
-  end
-
-  ##
-  # Declare that this gem requires ruby to be in the 1.9 family.
-
-  def ruby19!
-    require_ruby_version "~> 1.9"
-  end
-
   ##
   # Declare that this gem requires ruby to be in the 2.0+ family.
 
@@ -882,6 +911,9 @@ class Hoe
     require_ruby_version "~> 2.3"
   end
 
+  # I don't care for these methods (eg 3.0 is out) ... so I'm not
+  # continuing them.
+
   ##
   # Provide a linear degrading value from n to m over start to finis
   # dates. If not provided, start and finis will default to 1/1 and
@@ -907,6 +939,10 @@ class Hoe
     end
   end
 
+  def skip_intuit_values? # :nodoc:
+    %w[summary description homepage].all? { |field| send field }
+  end
+
   ##
   # Loads ~/.hoerc, merges it with a .hoerc in the current pwd (if
   # any) and yields the configuration and its path
@@ -915,37 +951,39 @@ class Hoe
     config = Hoe::DEFAULT_CONFIG
 
     rc = File.expand_path("~/.hoerc")
-    exists = File.exist? rc
-    homeconfig = exists ? YAML.load_file(rc) : {}
+    homeconfig = maybe_load_yaml rc
 
     config = config.merge homeconfig
 
     localrc = File.join Dir.pwd, ".hoerc"
-    exists = File.exist? localrc
-    localconfig = exists ? YAML.load_file(localrc) : {}
+    localconfig = maybe_load_yaml(localrc)
 
     config = config.merge localconfig
 
     yield config, rc
   end
-end
 
-class File
-  # Like File::read, but strips out a BOM marker if it exists.
-  def self.read_utf path
-    r19 = "<3".respond_to? :encoding
-    opt = r19 ? "r:bom|utf-8" : "rb"
-
-    open path, opt do |f|
-      if r19 then
-        f.read
+  def maybe_load_yaml path # :nodoc:
+    if File.exist? path then
+      if YAML.respond_to? :safe_load_file then
+        YAML.safe_load_file path, permitted_classes: [Regexp, Symbol]
       else
-        f.read.sub %r%\A\xEF\xBB\xBF%, ""
+        YAML.load_file path
       end
+    else
+      {}
     end
   end
 end
 
-def Gem.bin_wrapper name # :nodoc: HACK
-  File.join Gem.bindir, Gem.default_exec_format % name
-end unless Gem.respond_to? :bin_wrapper
+class File # :nodoc:
+  # Like File::read, but strips out a BOM marker if it exists.
+  def self.read_utf path
+    open path, "r:bom|utf-8" do |f|
+      f.read
+    end
+  end
+end
+
+module Gem # :nodoc:
+end
